import 'dart:io';

import 'package:dio/dio.dart';
import 'package:xh_framework/com/xh/networkapi/config/network_config.dart';
import 'package:xh_framework/com/xh/networkapi/exception/network_exception.dart';

typedef OnSuccessCallback<T> = void Function(T data);
typedef OnErrorCallback = void Function(NetworkException e);
typedef OnProgressCallback = void Function(int count, int total);
typedef OnAPIListener = void Function(
    {OnSuccessCallback onSuccess, OnErrorCallback onError});

const OK = 200;

abstract class HttpRequest {
  void getRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError});

  void postRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError});

  void uploadRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError});

  void downloadRequest(String url, String path,
      {OnSuccessCallback<String> onSuccess,
      OnProgressCallback onProgress,
      OnErrorCallback onError});
}

class XHHttpRequest implements HttpRequest {
  final Dio dio = new Dio();

  void _initConfig() {
    String proxyHostAndPort = NetworkConfig.getInstance().getProxyHostAndPort();
    if (proxyHostAndPort != null) {
      (dio.httpClientAdapter as DefaultHttpClientAdapter).onHttpClientCreate =
          (client) {
        client.badCertificateCallback =
            (X509Certificate cert, String host, int port) {
          return Platform.isAndroid;
        };
        client.findProxy = (url) {
          return 'PROXY $proxyHostAndPort';
        };
      };
    }
  }

  @override
  void getRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError}) async {
    try {
      _initConfig();
      Options options = new Options(
        connectTimeout: NetworkConfig.getInstance().getConnectTimeout(),
        sendTimeout: NetworkConfig.getInstance().getWriteTimeout(),
        receiveTimeout: NetworkConfig.getInstance().getReadTimeout(),
      );
      Response<T> response =
          await dio.get<T>(url, queryParameters: map, options: options);
      if (response.statusCode == OK) {
        onSuccess(response.data);
      }
    } catch (e) {
      NetworkException.formatException(e, onError);
    }
  }

  @override
  void postRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError}) async {
    try {
      _initConfig();
      Options options = new Options(
        connectTimeout: NetworkConfig.getInstance().getConnectTimeout(),
        sendTimeout: NetworkConfig.getInstance().getWriteTimeout(),
        receiveTimeout: NetworkConfig.getInstance().getReadTimeout(),
      );
      Response response =
          await dio.post<T>(url, queryParameters: map, options: options);
      if (response.statusCode == OK) {
        onSuccess(response.data);
      }
    } catch (e) {
      NetworkException.formatException(e, onError);
    }
  }

  @override
  void uploadRequest<T>(String url, Map<String, dynamic> map,
      {OnSuccessCallback<T> onSuccess, OnErrorCallback onError}) async {
    try {
      _initConfig();
      Options options = new Options(
        connectTimeout: NetworkConfig.getInstance().getConnectTimeout(),
        sendTimeout: NetworkConfig.getInstance().getWriteTimeout(),
        receiveTimeout: NetworkConfig.getInstance().getReadTimeout(),
      );

      FormData data = FormData();
      map.forEach((String key, dynamic value) {
        if (value is File) {
          File file = value;
          String fileName = file.path.substring(file.path.lastIndexOf("/") + 1);
          UploadFileInfo fileInfo = new UploadFileInfo(value, fileName);
          data.add(key, fileInfo);
        } else {
          data.add(key, value);
        }
      });
      Response response = await dio.post<T>(url, options: options, data: data);
      if (response.statusCode == OK) {
        onSuccess(response.data);
      }
    } catch (e) {
      NetworkException.formatException(e, onError);
    }
  }

  @override
  void downloadRequest(String url, String path,
      {OnSuccessCallback<String> onSuccess,
      OnProgressCallback onProgress,
      OnErrorCallback onError}) async {
    try {
      _initConfig();
      Options options = new Options(
        connectTimeout: NetworkConfig.getInstance().getConnectTimeout(),
        sendTimeout: NetworkConfig.getInstance().getWriteTimeout(),
        receiveTimeout: NetworkConfig.getInstance().getReadTimeout(),
      );

      Response response = await dio.download(url, path,
          options: options,
          onReceiveProgress: (count, total) => onProgress(count, total));
      if (response.statusCode == OK) {
        onSuccess(path);
      }
    } catch (e) {
      NetworkException.formatException(e, onError);
    }
  }
}
